<?php

namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\CADEST\Services\ComposicaoProdutoService;
use Teknisa\CADEST\Services\Ger03800ProdutosPorUnidadeService;
use Teknisa\Model\Util\NovoCodigo;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\Util\Functions;

class Ger14600ComposicaoDoProduto extends \Teknisa\CADEST\Controllers\TeknisaCrud {

    protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $service;
    protected $grupoPromocionalService;
    protected $utilities;

    public function __construct($crudService, $service, $service2, ComposicaoProdutoService $ComposicaoService, $grupoPromocionalService, Utilities $utilities) {
        $this->crudService = $crudService;
        $this->service = $service;
        $this->composicaoService = $ComposicaoService;
        $this->grupoPromocionalService = $grupoPromocionalService;
        $this->utilities = $utilities;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
            $this->dataSourceName = $dataSource;
            switch ($dataSource) {
                case 'produto_composicao':
                    $this->parameterBagColumns = array("CDFILIAL", "P_PRODINI", "P_PRODFIN", "CDPRODUTO", "CDNVPRODUTO", "IDTIPOCOMPPROD", "CDSEGMER", "TODOS", "NRCOMPPRODFLAG");
                    break;
                case 'rotfabrica_produto_departamento':
                    $this->parameterBagColumns = array("CDPRODUTO");
                    break;
                case 'rotfablocal_produto_departamento':
                    $this->parameterBagColumns = array("CDPRODUTO", "CDFILIAL");
                    break;
                case 'prodgrupromoc_produto':
                    $this->parameterBagColumns = array("CDGRUPROMOC");
                    break;
                case 'grupromocprod_grupopromoc':
                    $this->parameterBagColumns = array("CDPRODPROMOCAO");
                    break;
                case 'grupromocprfil_grupopromoc':
                    $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDFILIAL");
                    break;
                case 'desgrupromocpr_prodgrupromoc_produto':
                    $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDGRUPROMOC", "NRSEQGRPROMO");
                    break;
                case 'desgrupromprfil_prodgrupromoc_produto':
                    $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDGRUPROMOC", "CDFILIAL", "NRSEQGRPROMOFIL");
                    break;
            }

            parent::find($request, $response);
        }
        catch(\Exception $e) {
            throw $e;
        }
    }

    public function AltAuto(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[count($rows) - 1];
            $this->dataSourceName = "grupromocprod";

            foreach($row["CDGRUPROMOC"] as $key => $CDGRUPO) {
                $NRSEQGRPROMO = $row['NRSEQGRPROMO'][$key];
                
                if(!isset($row["NRORDPROMOGRUP"]) ){
                    $row["NRORDPROMOGRUP"] = "";
                }
                if(!isset($row["NMGRUPROMOC"]) ){
                    $row["NMGRUPROMOC"] = "";
                }
                if(!isset($row["CDGRUPMUTEX"]) ){
                    $row["CDGRUPMUTEX"] = "";
                }
                if(!isset($row["IDAPLICADESC"]) ){
                    $row["IDAPLICADESC"] = "N";
                }
                if(!isset($row["IDIMPGRPROMO"]) ){
                    $row["IDIMPGRPROMO"] = "S";
                }
                if(!isset($row["DTINIVGGRPROMO"]) ){
                    $row["DTINIVGGRPROMO"] = "";
                }
                if(!isset($row["DTFINVGGRPROMO"]) ){
                    $row["DTFINVGGRPROMO"] = "";
                }

                $params[0] = array (
                    'QTPRGRUPROMIN'     => $row['QTPRGRUPROMIN'],
                    'QTPRGRUPPROMOC'    => $row['QTPRGRUPPROMOC'],
                    'NRORDPROMOGRUP'    => $row['NRORDPROMOGRUP'],
                    'CDGRUPROMOC'       => $CDGRUPO,
                    'NMGRUPROMOC'       => $row['NMGRUPROMOC'],
                    'CDGRUPMUTEX'       => $row['CDGRUPMUTEX'],
                    'IDAPLICADESC'      => $row['IDAPLICADESC'],
                    'IDIMPGRPROMO'      => $row['IDIMPGRPROMO'],
                    'DTINIVGGRPROMO'    => $row['DTINIVGGRPROMO'],
                    'DTFINVGGRPROMO'    => $row['DTFINVGGRPROMO'],
                    'CDPRODPROMOCAO'    => $row['CDPRODPROMOCAO'],
                    '__is_new'          => false,
                    'NRSEQGRPROMO'      => $NRSEQGRPROMO
                );
                
                $newRequest = new DTO\Request\DataSet(
    	            new DataSet('grupromocprod_grupopromoc', $params),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                $newResponse = new DTO\Response();
                
                parent::save($newRequest, $newResponse);
            }
        }
        catch(\Exception $e) {
            var_dump($e->getMessage());
        }
    }

    public function AltAutoFil(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[count($rows) - 1];

            $this->dataSourceName = "grupromocprfil";

            foreach($row["CDGRUPROMOC"] as $key => $CDGRUPO) {
                $NRSEQGRPROMOFIL = $row['NRSEQGRPROMOFIL'][$key];
                
                if(!isset($row["NRORDPROMOGRUPFIL"]) ){
                    $row["NRORDPROMOGRUPFIL"] = "";
                }
                if(!isset($row["NMGRUPROMOC"]) ){
                    $row["NMGRUPROMOC"] = "";
                }
                if(!isset($row["CDGRUPMUTEXFIL"]) ){
                    $row["CDGRUPMUTEXFIL"] = "";
                }
                if(!isset($row["IDAPLICADESCFIL"]) ){
                    $row["IDAPLICADESCFIL"] = "N";
                }
                if(!isset($row["IDIMPGRPROMOFIL"]) ){
                    $row["IDIMPGRPROMOFIL"] = "S";
                }
                if(!isset($row["DTINIVGGRPROFIL"]) ){
                    $row["DTINIVGGRPROFIL"] = "";
                }
                if(!isset($row["DTFINVGGRPROFIL"]) ){
                    $row["DTFINVGGRPROFIL"] = "";
                }

                $params[0] = array (
                    'CDFILIAL'          => $row['CDFILIAL'],
                    'QTPRGRUPROMINFIL'  => $row['QTPRGRUPROMINFIL'],
                    'QTPRGRUPPROMFIL'   => $row['QTPRGRUPPROMFIL'],
                    'NRORDPROMOGRUPFIL' => $row['NRORDPROMOGRUPFIL'],
                    'CDGRUPROMOC'       => $CDGRUPO,
                    'NMGRUPROMOC'       => $row['NMGRUPROMOC'],
                    'CDGRUPMUTEXFIL'    => $row['CDGRUPMUTEXFIL'],
                    'IDAPLICADESCFIL'   => $row['IDAPLICADESCFIL'],
                    'IDIMPGRPROMOFIL'   => $row['IDIMPGRPROMOFIL'],
                    'DTINIVGGRPROFIL'   => $row['DTINIVGGRPROFIL'],
                    'DTFINVGGRPROFIL'   => $row['DTFINVGGRPROFIL'],
                    'CDPRODPROMOCAO'    => $row['CDPRODPROMOCAO'],
                    "NRORG"             => $row['NRORG'],
                    '__is_new'          => false,
                    "NRSEQGRPROMOFIL"   => $NRSEQGRPROMOFIL
                );
                $newRequest = new DTO\Request\DataSet(
    	            new DataSet('grupromocprfil_grupopromoc', $params),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                $newResponse = new DTO\Response();

                parent::save($newRequest, $newResponse);
            }
        }
        catch(\Exception $e) {
            var_dump($e->getMessage());
        }
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
            $rows = $request->getDataSet()->getRows();
            $rows = $rows[count($rows) - 1];
            self::moneyToFloat('QTSUBPRODLOC', $rows);
            self::moneyToFloat('QTPRGRUPROMINFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMOC', $rows);
            self::moneyToFloat('QTPRGRUPROMIN', $rows);
            self::moneyToFloat('QTSUBPRODROT', $rows);

            $this->dataSourceName = $dataSourceName;

            if($dataSourceName === 'rotfabrica_produto_departamento'
                || 'grupromocprod_grupopromoc'
                || 'desgrupromocpr_prodgrupromoc_produto'
                || 'rotfablocal_produto_departamento'
                || 'grupromocprfil_grupopromoc'
                || 'desgrupromprfil_prodgrupromoc_produto'
                || 'produtoadic_produto') {
                $this->dataSourceName = explode("_", $dataSourceName)[0];
            }

            if($dataSourceName === 'grupromocprfil_grupopromoc') {
                if(is_array($rows))
                    $rows = $rows[0];


                if(!Functions::arrayKeyExists('NRSEQGRPROMOFIL',  $rows) && !isset( $rows['NRSEQGRPROMOFIL']) && $rows['__is_new']) {
                    $rows->set('NRSEQGRPROMOFIL', NovoCodigo::geraCodigo($rows['NRORG'], "GRUPROMOCPRFIL".$rows['CDFILIAL'].$rows['CDPRODPROMOCAO'].$rows['CDGRUPROMOC'], 1, 10, false, false));
                }
            }
            if($dataSourceName === 'grupromocprod_grupopromoc') {
                if(is_array($rows)) {
                    $rows = $rows[0];
                }

                if(!Functions::arrayKeyExists('NRSEQGRPROMO', $rows) && !isset( $rows['NRSEQGRPROMO']) && $rows['__is_new']) {
                    $rows->set('NRSEQGRPROMO', NovoCodigo::geraCodigo($rows['NRORG'], "GRUPROMOCPROD".$rows['CDPRODPROMOCAO'].$rows['CDGRUPROMOC'], 1, 10, false, false));
                }

                $rows->set('QTPRGRUPPROMOC', floatval(str_replace(',', '.', $rows['QTPRGRUPPROMOC'])));
                $rows->set('QTPRGRUPROMIN',  floatval(str_replace(',', '.', $rows['QTPRGRUPROMIN'])));
            }

            if($dataSourceName === 'ger14600_composicao_do_produto') {
                if(is_array($rows))
                    $rows = $rows[0];
                $rows->set('__is_new', false);
            }
            
            $newRequest = new DTO\Request\DataSet (
	            new DataSet($dataSourceName, [$rows]),
	            $request->getMethod(),
	            $request->getRoutePath(),
	            $request->getUserId()
            );
            
            if($dataSourceName === 'rotfabrica_produto_departamento') {
                $resultado = $this->composicaoService->verificaComposicaoPadrao($rows);
                
                if($resultado) {
                    $response->setError(new DTO\Response\Error('Operação bloqueada. Período de vigência do item coincide com outro período de vigência cadastrado. Se o período de vigência não for informado, o item da composição estará sempre vigente.', 1));
                } else {
                    parent::save($newRequest, $response);
                }
            } else {
                parent::save($newRequest, $response);
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function updateGruposPromocionais($cdproduto, $cdfilial) {
        $this->grupoPromocionalService->updateImprCupom($cdproduto, $cdfilial);
    }

    public function saveProdGrupromoc(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
            $rows = $request->getDataSet()->getRows()[0];

            self::moneyToFloat('QTSUBPRODLOC', $rows);
            self::moneyToFloat('QTPRGRUPROMINFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMOC', $rows);
            self::moneyToFloat('QTPRGRUPROMIN', $rows);
            self::moneyToFloat('QTSUBPRODROT', $rows);

            $this->dataSourceName = $dataSourceName;

            if($dataSourceName === 'rotfabrica_produto_departamento'
                || 'grupromocprod_grupopromoc'
                || 'desgrupromocpr_prodgrupromoc_produto'
                || 'rotlocal_produto_departamento'
                || 'grupromocprfil_grupopromoc'
                || 'desgrupromprfil_prodgrupromoc_produto'
                || 'produtoadic_produto') {
                
                $this->dataSourceName = explode("_", $dataSourceName)[0];
            }

            if(Functions::arrayKeyExists('VRDESPRODPROMOC', $rows)){
                $validateDataProdGroup = self::validateDataProdGroupP($rows);
                 $request = new DTO\Request\DataSet (
    	            new DataSet($dataSourceName, [$rows]),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                 if($validateDataProdGroup){
                    parent::delete($request, $response);
                 } else {
                    parent::save($request, $response);
                 }
            }
            
            if(Functions::arrayKeyExists('VRDESPRPROMOFIL',  $rows)){
                $validateDataProdGroup = self::validateDataProdGroupL($rows);
                 if($validateDataProdGroup){
                    parent::delete($request, $response);
                 } else {
                    parent::save($request, $response);
                 }
            }

        }
        catch(\Exception $e) {
            throw $e;
        }
    }

    private function validateDataProdGroupP(&$rows){
        $retorno = false;
        if(!$rows['VRDESPRODPROMOC']){
            if($rows['IDAPLICADESCPR'] == 'T' && $rows['IDPRODPRESELEC'] == 'N' && $rows['IDPRINCPROD'] == 'N' && $rows['NRORDPROMOPR'] == null){
                $retorno = true;
            } else {
                $rows['VRDESPRODPROMOC'] = '0';
                $rows['IDDESCACRPROMO'] = 'D';
                $rows['IDPERVALORDES'] = 'V';
            }
        }
        return $retorno;
    }

    private function validateDataProdGroupL(&$rows){
        $retorno = false;
        if(!$rows['VRDESPRPROMOFIL']){
            if($rows['IDAPLICADESCPRFI'] == 'T' && $rows['IDPRODPRESELFIL'] == 'N' && $rows['IDPRINCPROD'] == 'N' && $rows['NRORDPROMOPRFIL'] == null){
                $retorno = true;
            } else {
                 $rows['VRDESPRPROMOFIL'] = '0';
                $rows['IDDESCACRPROFIL'] = 'D';
                $rows['IDPERVALDESFIL'] = 'V';
            }
        }
        return $retorno;
    }

    private function moneyToFloat($name, &$rows){
        if(Functions::arrayKeyExists($name, $rows)){
            if(gettype($rows[$name]) == 'string') {
                if (!!$rows[$name]) {
        			$rows[$name] = preg_replace("/[^0-9,]/", "", $rows[$name]);
        			$rows[$name] = str_replace ("," , "." , $rows[$name]);
        		} else {
        			$rows[$name] = 0;
        		}
            }
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        try{

            $rota = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 8);
            $data = explode("_", $rota);
            $dataSource = $data[0];
            $filhos = false;
            $row = array();
            $primaryKeys = array();

            if($dataSource === 'grupromocprfil' || $dataSource === 'grupromoc' || $dataSource === 'grupromocprod'){
                $filhos = true;
            }


            //---------------------------Informa quais são as tabelas com registros filhos-----------------------//
            if($dataSource === 'grupromoc') {
                $dataSourceFilhos = array(array('name' => 'prodgrupromoc', 'filhos' => 'false'),
                                          array('name' => 'grupromocprod', 'filhos' => 'false'),
                                          array('name' => 'grupromocprfil', 'filhos' => 'false'));
            }


            if($dataSource === 'prodgrupromoc') $dataSourceFilhos = array(array('name' => 'desgrupromocpr', 'filhos' => 'false'),
                                                                    array('name' => 'desgrupromprfil', 'filhos' => 'false'));

            if($dataSource === 'grupromocprod') $dataSourceFilhos = array(array('name' => 'desgrupromocpr', 'filhos' => 'false'));

            if($dataSource === 'grupromocprfil') $dataSourceFilhos = array(array('name' => 'desgrupromprfil', 'filhos' => 'false'));


            //-----------------------------------------------------------------------------------------------------//

            $row = $primaryKeys;
            if($filhos === true) {
               foreach($primaryKeys as $key) {
                    $requestGetRow = $request->getRow();
                    $row[$key] = $requestGetRow[$key];
               }


                $this->deleteFilhos($dataSourceFilhos, $primaryKeys, $row, $response);
            }

            $this->dataSourceName = $dataSource;
            parent::delete($request, $response);

        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteFilhos($dataSourceFilhos, $primaryKeys, $row, $response) {
    // Deleta Filhos

        // Percorre o vetor de dataSourceFilhos
        try {
            for($i =0; $i < count($dataSourceFilhos); $i++) {
                // Verifica se os filhos possuem filhos e chama o metado novamente caso tenham

                if($dataSourceFilhos[$i]['filhos'] === 'true') {
                    $this->deleteFilhos($dataSourceFilhos[$i]['dataSourceFilhos'], $primaryKeys, $row, $response);
                }

                $conditions = $this->crudService->getConditionsByColumns($primaryKeys, $row);

                $dataSet = $this->crudService->getFilteredDataSet($dataSourceFilhos[$i]['name'], $conditions);
                $this->crudService->delete($response, $dataSet);
            }
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveBlob(DTO\Request $request, DTO\Response $response) {

    	try{
            $row = $request->getRow();
            if (isset($row["TXROTFAB"])){
                    $row['__is_new'] = !$this->composicaoService->checkIsNewPadrao($row);
                    $row["TXROTFAB"] = bin2hex($row["TXROTFAB"]);
                    $this->dataSourceName = "descroteiro";
            } else {
                if (isset($row["TXROTFABLOC"])){
                    $row['__is_new'] = !$this->composicaoService->checkIsNewLocal($row);
                    $row["TXROTFABLOC"] = bin2hex($row["TXROTFABLOC"]);
                    $this->dataSourceName = "descrotlocal";
                } else {
                    throw Exception("Row passada não possui campo BLOB");
                }
            }
            $newRequest = new DTO\Request\Row ($row, '', '', '');
    	    parent::save($newRequest, $response);
    	} catch(\Exception $e){
    	    $response->setError(new DTO\Response\Error($e->getMessage(),1));
    	}

    }

    public function importComp(DTO\Request $request, DTO\Response $response){
        $rows = $request->getRow();
        if($rows['hasComp']){
            //deleta composição
        }
    }

    public function findProductsWithComp(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->parameterBagColumns = array("NRORG");
            $this->dataSourceName = 'produto_composicao_PadraoLocal';
            parent::find($request, $response);
        }
        catch(\Exception $e) {
            throw $e;
        }

    }

    public function checkForRecursiveComposition(DTO\Request\Row $request, DTO\Response $response) {
        // 'CDPRODUTOPAI' -> Código do produto pai.
        // 'CDPRODUTOFILHO' -> Código do produto filho.
        $row = $request->getRow();
        $this->compositionProducts = array();

        $produtosFilhos = array_merge_recursive(array($row['CDPRODUTOFILHO']), array());

        foreach($produtosFilhos as $cdprodutofilho) {
            $row['CDPRODUTOFILHO'] = $cdprodutofilho;
            $this->buildCompositionArray($request, $row);
        }

        $check = $this->isARecursiveComposition($row['CDPRODUTOPAI']);

        unset( $this->compositionProducts );

        $response->addDataSet( new DataSet ('isRecursiveComposition', [$check]) );
    }

    private function isARecursiveComposition ( $cdproduto ) {
        return in_array($cdproduto, $this->compositionProducts);
    }

    private function genericFindComposition ($request, $row) {
        $products = array();

        $newRequest = $this->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $row);

        $response = new Response ();

        parent::find($newRequest, $response);

        $dataSets = $response->getDataSets();

        if(count($dataSets)) {
            $products = $dataSets[0]->getRows();
        }

        return $products;
    }

    private function getCompositionItems ($request, $row) {
        $products = array();

        if(!Functions::arrayKeyExists('CDFILIAL', $row)){
            $composicaoPadraoFilter['CDPRODUTO'] = $row['CDPRODUTOFILHO'];

            $this->parameterBagColumns = array("CDPRODUTO");
            $this->dataSourceName = 'rotfabrica_produto_departamento';

            $products = array_merge($products, $this->genericFindComposition($request, $composicaoPadraoFilter));
        } else {
            $composicaoLocalFilter['CDPRODUTO'] = $row['CDPRODUTOFILHO'];
            $composicaoLocalFilter['CDFILIAL']  = $row['CDFILIAL'];

            $this->dataSourceName = 'rotfablocal_produto_departamento';
            $this->parameterBagColumns = array("CDPRODUTO", "CDFILIAL");

            $products = array_merge($products, $this->genericFindComposition($request, $composicaoLocalFilter));
        }

        return $products;
    }

    private function buildCompositionArray ($request, $row) {
        $products = $this->getCompositionItems($request, $row);

        if(count($products) === 0 ) {
            array_push($this->compositionProducts, $row['CDPRODUTOFILHO']);
        } else {
            foreach ($products as $product) {
                array_push($this->compositionProducts, $product['CDPRODUTO']);
                if(!empty($product['CDSUBPRODLOC']) && !$this->isARecursiveComposition($product['CDSUBPRODLOC']) ) {
                    $row['CDPRODUTOFILHO'] = $product['CDSUBPRODLOC'];
                    $this->buildCompositionArray($request, $row);
                }
                if(!empty($product['CDSUBPRODUTO']) && !$this->isARecursiveComposition($product['CDSUBPRODUTO'])) {
                    $row['CDPRODUTOFILHO'] = $product['CDSUBPRODUTO'];
                    $this->buildCompositionArray($request, $row);
                }
            }
        }
    }

    private function createNewZeedhiFilterRequestObject($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }

        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                1,
                500
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }
}